/**
 * main class extension
 */
const app = (function () {
  const api = {
    wiservice: null,
    collaborationConnected: false,
    extensionEnabled: true,
    calls: {},
    removedCalls: {},
    tabs: {},
    collaboration: [],
    defRedirectUrl: "https://kite.wildix.com/api/v1/InfusionsoftAuthSuccess",
    reconnectDelay: 1000,
    timeoutMultiplier: 1,
    timeoutMaxMultiplier: 10,
    idForSearchContact: null,
    countryTel: null,
    sw: null,
    messageWiService: {
      'message': 'M_WISERVICE',
      'msgdata': {
        'command': 'getconnectionstatus'
      }
    },
    init: function () {
      this.subscribeWiservice();
      this.eventsChrome();
      this.getCountryTel.bind(this)();
      this.installSW.bind(this)();
      chrome.storage.local.get('extensionStatus', function (enabled) {
        if (enabled.extensionStatus !== undefined) {
          enabled = enabled.extensionStatus;
        } else {
          enabled = true;
        }
        if (enabled === true) {
          chrome.browserAction.setIcon({path: chrome.extension.getURL("images/is-16.png")});
        } else {
          chrome.browserAction.setIcon({path: chrome.extension.getURL("images/is-16-disabled.png")});
        }
        this.extensionEnabled = enabled;
      });

    },
    subscribeWiservice: function () {
      this.wiservice = null;
      delete this.wiservice;
      this.wiservice = new Wildix.IntegrationService({
        name: "WildixInfusionsoftIntegration",
        version: '1.0.0'
      });
      this.wiservice.on('collaboration:connected', function (info) {
        console.log('Collaboration connected', info);
        this.collaborationConnected = true;
        this.clearCalls();
        this.collaboration = info;
        this.sendMessageToAllTabs("CLOSE_COLLABORATION_NOT_OPENED_MESSAGE");
        this.sendMessageToAllTabs("CLOSE_WISERVICE_CONNECTION_ERROR_MESSAGE");
      }.bind(this));
      this.wiservice.on('collaboration:disconnected', function (info) {
        console.log('Collaboration disconnected', info);
        this.collaborationConnected = false;
        this.clearCalls();
        this.sendMessageToAllTabs("SHOW_COLLABORATION_NOT_OPENED_MESSAGE");
        this.sendMessageToAllTabs("CLOSE_WISERVICE_CONNECTION_ERROR_MESSAGE");
      }.bind(this));
      this.wiservice.on('connection:error', function (info) {
        console.warn('WIService not installed or is not running', info);
        this.sendMessageToAllTabs("SHOW_WISERVICE_CONNECTION_ERROR_MESSAGE");
        setTimeout(function () {
          if (this.timeoutMultiplier < this.timeoutMaxMultiplier) {
            this.timeoutMultiplier++;
          }
          this.subscribeWiservice();
        }.bind(this), this.reconnectDelay * this.timeoutMultiplier);
        this.clearCalls();
      }.bind(this));
      this.wiservice.on('ready', function (info) {
        console.info('WIService is ready', info);
        this.timeoutMultiplier = 1;
        this.clearCalls();
        this.sendMessageToAllTabs("CLOSE_WISERVICE_CONNECTION_ERROR_MESSAGE");
      }.bind(this));
      this.wiservice.Telephony.on('add', this.onCallAdd.bind(this));
      this.wiservice.Telephony.on('remove', this.onCallRemove.bind(this));
      this.wiservice.on('updatecall', this.updateCall.bind(this));
      this.wiservice.Telephony.subscribe();
    },
    eventsChrome: function () {
      chrome.runtime.onMessage.addListener(this.onMessage.bind(this));
      chrome.tabs.onRemoved.addListener(this.onTabRemove.bind(this));
      chrome.webRequest.onCompleted.addListener(function (details) {
        this.sendMessageToTab(details.tabId, "PARSE_PHONE_NUMBERS_SEARCH_PAGE");
      }.bind(this), {urls: ["*://*.infusionsoft.com/*search*"], types: ['xmlhttprequest']});
      chrome.webRequest.onCompleted.addListener(function (details) {
        this.sendMessageToTab(details.tabId, "PARSE_PHONE_NUMBERS_CONTACT_PAGE");
      }.bind(this), {urls: ["*://*.infusionsoft.com/Contact*"], types: ['xmlhttprequest']});
    },
    clearCalls: function () {
      for (let account in this.calls) {
        this.calls[account] = {};
      }
    },
    sendMessageToAllTabs: function (type, data, callback) {
      if (this.extensionEnabled === true) {
        for (let host in this.tabs) {
          this.sendMessageToHostTabs(host, type, data, callback);
        }
      }
    },
    sendMessageToHostTabs: function (host, type, data, callback) {
      if (this.tabs[host]) {
        this.tabs[host].forEach(function (tabId) {
          chrome.tabs.get(tabId, function (tab) {
            if (typeof tab !== 'undefined') { // it may be a closed tab
              this.sendMessageToTab(tab.id, type, data, callback);
            }
          }.bind(this));
        }.bind(this));
      }
    },
    sendMessageToTab: function (tabId, type, data, callback) {
      if (this.extensionEnabled === true) {
        data = data || {};
        data['messageType'] = type;
        chrome.tabs.sendMessage(tabId, data, callback);
      }
    },
    showIntegrationInvitation: function () {
      this.sendMessageToAllTabs("SHOW_INTEGRATION_INVITE");
    },
    showIntegrationInvitationAuthFailed: function () {
      this.sendMessageToAllTabs("SHOW_INTEGRATION_INVITE_AUTH_FAILED");
    },
    onTokenFail: function () {
      this.showIntegrationInvitationAuthFailed();
    },
    onTabRemove: function (tabId, info) {
      for (let account in this.tabs) {
        this.tabs[account].forEach(function (existingTabId, index) {
          if (tabId === existingTabId) {
            this.tabs[account].splice(index, 1);
          }
        }.bind(this));
      }
    },
    onMessage: function (request, sender, sendResponse) {
      let host = '';
      if (sender.tab !== undefined) {
        host = this.parseUrl(sender.tab.url)['hostname'];
      }
      console.log('chrome.runtime.onMessage ', request.messageType, host);
      switch (request.messageType) {
        case 'REGISTER':
          // when tab is reloading with another host, we need to remove old record from this.tabs array
          this.onTabRemove(sender.tab.id, null);
          if (this.tabs[host] === undefined) {
            this.tabs[host] = [];
          }
          if (this.tabs[host].indexOf(sender.tab.id) === -1) {
            this.tabs[host].push(sender.tab.id);
          }
          sendResponse({messageType: 'REGISTER', messageText: 'OK'});
          this.checkIntegration();
          if (this.collaborationConnected === false) {
            this.sendMessageToAllTabs("SHOW_COLLABORATION_NOT_OPENED_MESSAGE");
          }
          break;
        case 'GET_REDIRECT_URI':
          this.getRedirectUri().then(val => {
            sendResponse(val);
          });
          break;
        case 'CLOSING_POPUP':
          this.sendMessageToHostTabs(host, "CLOSE_POPUP", {contactId: request.contactId});
          break;
        case 'SAVE_NEW_NOTE':
          console.log('SAVE_NEW_NOTE', host, request.contactId, request.note);
          this.addNote(host, request.contactId, request.note).then(promiseResult => {
            console.log('note promise success');
            this.sendMessageToHostTabs(host, "RECEIVE_NEW_NOTE", {contactId: request.contactId, note: request.note});
          }).catch(this.catchTokenProblemError.bind(this))
            .catch(message => {
              console.error('could not add note', host, request, message);
            });
          break;
        case 'GET_NOTES_BY_CONTACT':
          this.getNotes(host, request.contactId, function (contactId, tabId, listNotes) {
            this.sendMessageToTab(tabId, 'GET_NOTES_BY_CONTACT', {listNotes: listNotes, contactId: contactId});
          }.bind(this, request.contactId, sender.tab.id));
          break;
        case 'CREATING_CONTACT':
          console.log(request.removeCall);
          if (request.removeCall) {
            request.contact.state = 'remove';
          } else {
            request.contact.state = this.searchCallById(request.contact.callId).state;
          }
          this.creatingContact(request.contactId, request.contact, host, sender.tab.id);
          break;
        case 'MAKE_CALL':
          let number = request.number;
          this.idForSearchContact = request.idContact;
          if (typeof number !== 'undefined') {
            console.log('MAKE_CALL', number);
            this.wiservice.Telephony.call(number.toString(), function (result) {
              console.log('call callback', result);
            });
          }
          break;
        case 'START_INTEGRATION':
          this.getRedirectUri().then(val => {
            window.oauth2.requestPermission(val);
          });
          break;
        case 'CLOSE_INTEGRATION_INVITATION_POPUP':
          this.sendMessageToAllTabs("CLOSE_INTEGRATION_INVITATION_POPUP");
          break;
        case 'SHOWING_INTEGRATION_INVITE' :
          this.showIntegrationInvitation();
          break;
        case 'CHECK_COLLABORATION_OPENED':
          sendResponse({farewell: this.collaborationConnected});
          break;
        case 'ENABLE_EXTENSION':
          this.extensionEnabled = true;
          break;
        case 'DISABLE_EXTENSION':
          this.extensionEnabled = false;
          break;
        case 'GET_HOST':
          const arrHost = [];
          for (let tab in this.tabs) {
            arrHost.push(tab);
          }
          sendResponse({host: arrHost});
          break;
        case 'GET_CALLS':
          if (this.calls[host] === undefined) {
            this.calls[host] = {};
          }
          sendResponse({CALLS: this.calls[host], notes: [], removedCalls: this.removedCalls});
          break;
        case 'ACCEPT_CALL':
          this.wiservice.Telephony.answer(request.channel);
          break;
        case 'DECLINE_CALL':
          this.wiservice.Telephony.hangup(request.channel);
          break;
        case 'PAUSE_CALL':
          this.wiservice.Telephony.hold(request.channel);
          break;
        case 'REMOVE_PAUSE_CALL':
          this.wiservice.Telephony.unhold(request.channel);
          break;
        case 'GET_COLLABORATION_VERSION':
          this.getCollaborationVersion().then(version => {
            sendResponse(version);
          }).catch(e => {
            console.error(e);
          });
          break;
        case 'GET_CONTACT_BY_ID':
          for (let account in this.tabs) {
            this.getContactByID(account, request.id, 'Id').then(contactResponse => {
              sendResponse(contactResponse);
            }).catch(message => {
              console.error(message);
            });
          }
          break;
      }
      return true
    },
    getRedirectUri: function () {
      let redirect_url = this.defRedirectUrl;
      const getUrl = () => {
        return new Promise((resolve, reject) => {
          this.collaboration.getConnection().send(this.messageWiService, function (response) {
            const pbx = response.connected.collaboration[0].pbx;
            if (pbx && 'hostname' in pbx) {
              redirect_url = "https://" + pbx.hostname + (pbx.port ? ":" + pbx.port : "") + "/api/v1/InfusionsoftAuthSuccess";
              resolve(redirect_url);
            } else {
              reject(new Error('Failed get redirect url'));
            }
          });
        })
      };
      return getUrl().then(v => v);
    },
    getCollaborationVersion: function () {
      const getVersion = () => {
        return new Promise((resolve, reject) => {
          this.collaboration.getConnection().send(this.messageWiService, function (response) {
            console.log('collaboration status', response);
            const version = response.connected.collaboration[0].version;
            if (version) {
              resolve(version);
            } else {
              reject(new Error('Failed get collaboration version'));
            }
          });
        })
      };
      return getVersion().then(v => v);
    },
    checkIntegration: function () {
      // check if at least one integration is set
      chrome.storage.local.get('tokens', function (tokens) {
        if (tokens.tokens !== undefined) {
          tokens = tokens.tokens;
          if (Object.keys(tokens).length === 0 && tokens.constructor === Object) {
            this.showIntegrationInvitation();
          }
        } else {
          this.showIntegrationInvitation();
        }
      }.bind(this));
    },
    infusionsoftOldAPIRequest: function (account, methodName, params) {
      return infusionsoftAPI.getValidAccessToken(account)
        .then(accessToken => infusionsoftAPI.xmlrpcWithRetry(accessToken, methodName, params));
    },
    infusionsoftNewAPIRequest: function (account, endpoint, data) {
      return infusionsoftAPI.getValidAccessToken(account)
        .then(accessToken => infusionsoftAPI.ajaxWithRetry(accessToken, endpoint, data));
    },
    catchTokenProblemError: message => {
      switch (message) {
        case infusionsoftAPI.refreshTokenIsExpiredMessage:
        case infusionsoftAPI.noValidTokenStoredMessage:
          this.onTokenFail();
          break;
      }
    },
    addNote: function (account, contactId, note) {
      return infusionsoftAPI.getUserId(account).then(userId => {
        return this.infusionsoftOldAPIRequest(
          account,
          'DataService.add',
          [
            window.oauth2.client_id,
            'ContactAction',
            {
              'ContactId': contactId,
              'ActionDescription': note.ActionDescription,
              'CreationNotes': note.CreationNotes,
              'UserID': userId,
              'CreatedBy': userId,
              'ActionDate': (new Date).toISOString(), // we need to send this field for the record to become a note
              'CompletionDate': (new Date).toISOString(), // we need to send this field for the record to become a note
            }
          ]
        );
      });
    },
    getNotes: function (account, contactId, callback) {
      this.infusionsoftOldAPIRequest(
        account,
        'DataService.query',
        [
          window.oauth2.client_id,
          'ContactAction',
          100,
          0,
          {
            'ContactId': contactId,
            'ObjectType': 'Note'
          },
          ['ActionDate', 'ActionType', 'CreationDate', 'CreationNotes', 'ActionDescription', 'CompletionDate', 'LastUpdated', 'LastUpdatedBy', 'Accepted', 'CreatedBy', 'Id'],
        ]
      ).then(notesResponse => {
        callback(notesResponse[0] || []);
      }).catch(this.catchTokenProblemError.bind(this))
        .catch(message => {
          console.error('could not get notes', account, contactId, message);
      });
    },
    creatingContact: function (contactId, data, host, fromTabId) {
      console.log('CREATING CONTACT', data);
      let contact = {
        phone_numbers: [{
          field: "PHONE1",
          number: data.Phone1
        }]
      };
      if (data.FirstName !== undefined && data.FirstName !== '') {
        contact.given_name = data.FirstName;
      }
      if (data.LastName !== undefined && data.LastName !== '') {
        contact.family_name = data.LastName;
      }
      infusionsoftAPI.getUserId(host).then(userId => {
        contact.owner_id = userId;
        this.infusionsoftNewAPIRequest(
          host,
          '/contacts',
          JSON.stringify(contact)
        ).then(contactResponse => {
          data.Id = contactResponse.id;
          data.link = 'https://' + host + '/Contact/manageContact.jsp?view=edit&ID=' + data.Id;
          data.Company = '';
          console.log('CREATING CONTACT', data, this.searchCallById(contact.callId), );
          if (this.calls[host] !== undefined) {
            this.calls[host][data.callId.toString()] = data;
          }
          this.sendMessageToHostTabs(host, "ADD_CONTACT_INFO", {contactId: contactId, contact: data});
          chrome.tabs.create({url: 'https://' + host + '/Contact/manageContact.jsp?view=edit&ID=' + contactResponse.id});
        }).catch(this.catchTokenProblemError.bind(this))
          .catch(message => {
            console.error('could not create contact', host, contact, message);
          });
      });
      return true;
    },
    findCountry: function (number) {
      return this.countryTel.find(c => {
        let n = number.split('');
        n.length = n[0] === '+' ? c.dial_code.length : c.dial_code.length - 1;
        n = n.join('');
        return n === c.dial_code || n === c.dial_code.replace('+', '');
      }) || null
    },
    onCallAdd: async function (call) {
      let number = call.attributes.number;
      const country = this.findCountry(number);
      console.log('number', number);
      console.log('call', call);
      console.log('country', country);
      console.log('idForSearchContact', this.idForSearchContact);
      for (let account in this.tabs) {
        try {
          let contactResponse;
          // debugger;
          if (this.idForSearchContact == null) {
            contactResponse = await this.getContactByPhone(account, number);
          } else {
            contactResponse = await this.getContactByID(account, this.idForSearchContact, 'Id');
          }
          console.log('Tel have county code?', contactResponse, this.idForSearchContact, country);
          if ((contactResponse == null || typeof contactResponse === 'undefined') &&
            this.idForSearchContact == null &&
            country !== null) {
            const withoutCountryCodeNumber = number.replace(country.dial_code, '');
            console.log(withoutCountryCodeNumber);
            contactResponse = await this.getContactByPhone(account, withoutCountryCodeNumber);
            console.log('CONTACT RESPONSE', contactResponse);
          }
          if (contactResponse !== null) {
            await this.callWithContact(call, account, contactResponse);
            return false
          }
          throw infusionsoftAPI.noContactIsFoundMessage;
        } catch (message) {
          this.catchTokenProblemError(message);
          switch (message) {
            case infusionsoftAPI.noContactIsFoundMessage:
              await this.callWithNewContact(call, account);
              break;
            default:
              console.error('could not find contact for call', call, account, message);
              await this.callWithNewContact(call, account);
              break;
          }
        }
      }
    },
    getContactByPhone: async function (account, callNumber) {
      console.log('getContactByPhone', account, callNumber);
      for(let i = 0; i < 5; i++) {
        const indexPhone = (i + 1);
        const res = await this.findContactByField(account, callNumber, 'Phone' + indexPhone);
        console.log('Phone: ' + indexPhone);
        console.log('Result: ', res);
        if (res !== null) {
          res.indexPhone = indexPhone;
          return res
        }
        if (i === 5) {
          return null
        }
      }
    },
    getContactByID: function (account, data, fieldName) {
      return this.findContactByField(account, data, fieldName, true);
    },
    findContactByField: function (account, dataSearch, fieldName, additionalSetting) {
      // let phone = '+49 89 450 611 0';
      // Numbers can be in different formats with spaces and hyphens
      // therefore we search by figures
      let phone = dataSearch.replace(/\D/g, '');
      let numbers = phone.split('');
      let phoneForSearch = (numbers.length > 5 ? '%' : '') + numbers.join('%');
      return this.infusionsoftOldAPIRequest(
        account,
        'DataService.findByField',
        [
          window.oauth2.client_id,
          'Contact',
          10,
          0,
          fieldName,
          additionalSetting ? dataSearch : phoneForSearch,
          ['Id', 'Company', 'FirstName', 'LastName', fieldName, 'Email', 'Phone1', 'Phone2', 'Phone3', 'Phone4', 'Phone5'],
        ]
      ).then(contactsResponse => {
        if (additionalSetting) {
          const concatArray = arr1 => {
            return arr1.reduce((acc, val) => Array.isArray(val) ? acc.concat(concatArray(val)) : acc.concat(val), []);
          };
          return concatArray(contactsResponse).find(item => item.Id.toString() === dataSearch);
        }
        let len = contactsResponse[0].length;
        if (len > 0) { // contact is found
          let phoneNumbers = false;
          for (let i = 0; i < len; i++) {
            phoneNumbers = contactsResponse[0][i][fieldName].replace(/\D/g, '');
            // only an exact match
            if (phoneNumbers === phone) {
              return contactsResponse[0][i];
            }
          }
        }
        return null
      });
    },
    callWithNewContact: function (call, host) {
      console.log('contact is not found');
      const atrCall = call.attributes;
      const phone = atrCall.number;
      let names = atrCall.name.split(' ');
      let firstName = names.length > 0 ? names.splice(0, 1)[0] : '';
      let secondName = names.join(' ');
      let contact = {
        callId: call.id,
        callType: atrCall.direction,
        FirstName: firstName,
        LastName: secondName,
        Phone1: atrCall.number,
        Company: '',
        wmsPhone: atrCall.number,
        state: atrCall.state,
        Id: 'new' + Math.round(Math.random() * 10000),
      };
      if (this.calls[host] === undefined) {
        this.calls[host] = {};
      }
      if (this.removedCalls[call.id.toString()] === undefined) {
        this.calls[host][call.id.toString()] = contact;
        console.log('ADDED CALL', call.id, this.calls);
        console.log('contact is not found', call, contact);
        this.createNotification(
          chrome.runtime.getURL('images/is-128.png'),
          atrCall.name,
          contact.callType,
          atrCall.channel
        );
        this.sendMessageToHostTabs(host, "SHOW_SMALL_POPUP_NEW", {contact, phone});
      }
    },
    callWithContact: function (call, host, contact) {
      const phone = contact['Phone' + contact.indexPhone];
      contact.callId = call.id;
      contact.callType = call.attributes.direction;
      contact.wmsPhone = call.attributes.number;
      contact.state = call.attributes.state;
      console.log('contact found', contact, call.attributes);
      contact.link = 'https://' + host + '/Contact/manageContact.jsp?view=edit&ID=' + contact.Id;
      if (contact.Company === undefined) {
        contact.Company = '';
      }
      if (contact.FirstName === undefined) {
        contact.FirstName = '';
      }
      if (contact.LastName === undefined) {
        contact.LastName = '';
      }
      if (this.calls[host] === undefined) {
        this.calls[host] = {};
      }
      if (this.removedCalls[call.id.toString()] === undefined) {
        this.calls[host][call.id.toString()] = contact;
        console.log('ADDED CALL', call.id, this.calls);
        console.log('contact found', call, contact);

        this.createNotification(
          chrome.runtime.getURL('images/is-128.png'),
          contact.FirstName + ' ' + contact.LastName,
          contact.callType,
          call.attributes.channel,
          contact.link
        );
        this.sendMessageToHostTabs(host, "SHOW_SMALL_POPUP_EXISTING", {contact, phone});
      }
    },
    onCallRemove: function (call) {
      this.idForSearchContact = null;
      this.removedCalls[call.id.toString()] = call;
      for (let account in this.calls) {
        if (this.calls[account][call.id.toString()] !== undefined) {
          delete this.calls[account][call.id.toString()];
          console.log('REMOVED CALL', call.id, this.calls);
        }
      }
      this.sendMessageToAllTabs('HIDE_SMALL_POPUP', {callId: call.id, call});
    },
    updateCall: function (call) {
      const localCall = this.searchCallById(call.channel);
      if (localCall !== null) {
        localCall.state = call.state
      }
      console.log('CALLS AFTER UPDATE CALLS', this.calls, call);
      this.sendMessageToAllTabs('UPDATE_CALL', {call});
    },
    parseUrl: function (url) {
      const a = document.createElement('a');
      a.href = url;
      return (a || {});
    },
    getCountryTel: function () {
      $.get(chrome.runtime.getURL('libs/data/country_tel.json'), (data, status) => {
        if (status === 'success') {
          this.countryTel = data;
        }
      });
    },
    installSW: async function () {
      try {
        this.sw = await window.isw();
        navigator.serviceWorker.addEventListener('message', this.onMessageFromSw.bind(this))
      } catch (e) {
        console.error(e)
      }
    },
    createNotification: function (icon, title, callType, channel, link) {
      const data = {
        icon,
        title,
        callType,
        channel,
        link: link || null
      };
      this.sendMessageToSw(data)
    },
    sendMessageToSw: function (message) {
      if (message) {
        navigator.serviceWorker.controller.postMessage(message);
      }
    },
    onMessageFromSw: function (event) {
      const channel = event.data.data.channelCall;
      if (event.data.link === null) {
        return false;
      }
      switch (event.data.action) {
        case 'answer-call':
          this.wiservice.Telephony.answer(channel);
          for(let host in this.tabs) {
            this.sendMessageToTab(this.tabs[host][0], 'OPEN_CONTACT_WINDOW', event.data)
          }
          break;
        case 'reject-call':
          this.wiservice.Telephony.hangup(channel);
          break;
        case 'click':
          for(let host in this.tabs) {
            this.sendMessageToTab(this.tabs[host][0], 'OPEN_CONTACT_WINDOW', event.data)
          }
          break;
      }
    },
    searchCallById: function (id) {
      const _calls = this.calls;
      for (let host in _calls) {
        if (typeof _calls[host][id] !== 'undefined') {
          return _calls[host][id]
        }
        return null
      }
    }
  };
  return api;
})();
