window.addEventListener("load", function () {
  chrome.runtime.onInstalled.addListener(function (details) {
    console.log('chrome.runtime.onInstalled.addListener', chrome.management);

    // fix load content script in an already open tabs after update or install extension
    if (details.reason === 'update' || details.reason === 'install') {
      injectScripts();
    }

    app.init();
  });

  chrome.runtime.onStartup.addListener(function () {
    console.log('chrome.runtime.onStartup');
    app.init();
  });
});

/**
 * parse manifest.json
 * inject script by config
 */

let injectScripts = function () {
  var manifest = chrome.runtime.getManifest();
  var cs = [];
  for (var i = 0; i < manifest['content_scripts'].length; i++) {
    cs[i] = {
      'matches': new RegExp('(' + manifest['content_scripts'][i]['matches'].join(')|(').replace(/\*/g, '.*') + ')'),
      'js': manifest['content_scripts'][i]['js']
    }
    if (manifest['content_scripts'][i]['exclude_matches']) {
      cs[i]['exclude_matches'] = new RegExp('(' + manifest['content_scripts'][i]['exclude_matches'].join(')|(').replace(/\*/g, '.*') + ')');
    }
  }
  var cs_count = cs.length;
  chrome.windows.getAll({populate: true}, function (allWindows) {
    var url;
    for (var i in allWindows) {
      for (var j in allWindows[i].tabs) {
        url = allWindows[i].tabs[j].url;
        for (var k = 0; k < cs_count; k++) {
          if (url.search(cs[k]['matches']) != -1 && (!cs[k]['exclude_matches'] || url.search(cs[k]['exclude_matches']) == -1)) {
            cs[k]['js'].forEach(function (js) {
              chrome.tabs.executeScript(allWindows[i].tabs[j].id, {file: js});
            });
          }
        }
      }
    }
  });
};
