
const checkCollaboration = () => {
  const promise = () => {
    return new Promise((res, rej) => {
      chrome.runtime.sendMessage({messageType: 'CHECK_COLLABORATION_OPENED'}, function (result) {
        if (result) {
          res(result)
        } else {
          rej('failed check collaboration opened')
        }
      })
    })
  };
  return promise().then(v => v)
};
const initPopUp = () => {
  checkCollaboration().then(response => {
    if (response.farewell === false) {
      $('.wildix-integrations-wrapper').hide();
      $('.control-enable').hide();
      $('.control-disable').hide();
      $('.error-wrapper').show();
      this.sendMessageToAllTabs("SHOW_COLLABORATION_NOT_OPENED_MESSAGE");
    } else {
      $('.error-wrapper').hide();
      $('.wildix-integrations-wrapper').show();
      let showIntegrations = function () {
        chrome.storage.local.get('tokens', function (tokens) {
          if (tokens.tokens !== undefined) {
            tokens = tokens.tokens;

            let activeIntegrationsContainer = $('<div class="active-integrations-container"></div>');
            if (Object.keys(tokens).length !== 0 && tokens.constructor === Object) {
              activeIntegrationsContainer.append('<div class="integrations-title">Active integrations</div>');

              let timestamp = Math.round(new Date() / 1000);
              for (let account in tokens) {
                if (!tokens.hasOwnProperty(account)) {
                  continue;
                }
                let integrationWarning = '';
                if ((tokens[account].refresh_token_expires_in - timestamp) < 0) {
                  integrationWarning = '<div class="integration-warning"><img src="images/warning-16.png"/></div>';
                }
                let integrationDiv = $('<div class="active-integration">' + integrationWarning + '<div class="integration-title">' + account + '</div><div class="remove-integration" data-account="' + account + '"><img src="images/cross.png"/></div></div>');

                activeIntegrationsContainer.append(integrationDiv);
              }
            }
            $('.active-integrations-container').replaceWith(activeIntegrationsContainer);
          }
        });
      };

      let checkIntegrations = function (tokens) {
        if (Object.keys(tokens).length === 0 && tokens.constructor === Object) {
          chrome.runtime.sendMessage({
            messageType: 'SHOWING_INTEGRATION_INVITE'
          });
        }
      };

      let toggleExtensionControlButtons = function (enabled) {
        $('.control-enable').toggle(!enabled);
        $('.control-disable').toggle(enabled);
      };

      let enableExtension = function () {
        chrome.storage.local.set({'extensionStatus': true}, function () {
          chrome.browserAction.setIcon({path: chrome.extension.getURL("images/is-16.png")});

          toggleExtensionControlButtons(true);

          chrome.runtime.sendMessage({
            messageType: 'ENABLE_EXTENSION'
          });

          chrome.storage.local.get('tokens', function (tokens) {
            if (tokens.tokens !== undefined) {
              tokens = tokens.tokens;

              checkIntegrations(tokens);
            }
          });
        });
      };

      let disableExtension = function () {
        chrome.storage.local.set({'extensionStatus': false}, function () {
          chrome.browserAction.setIcon({path: chrome.extension.getURL("images/is-16-disabled.png")});

          toggleExtensionControlButtons(false);

          chrome.runtime.sendMessage({
            messageType: 'DISABLE_EXTENSION'
          });
        });
      };

      showIntegrations();

      chrome.storage.local.get('extensionStatus', function (enabled) {
        if (enabled.extensionStatus !== undefined) {
          enabled = enabled.extensionStatus;
        } else {
          enabled = true;
        }
        toggleExtensionControlButtons(enabled);
      });

      $(document).on('click', '.wildix-integrations-wrapper .add-new-integration', function () {
        chrome.runtime.sendMessage({
          messageType: 'START_INTEGRATION'
        });
      });

      $(document).on('click', '.wildix-integrations-wrapper .remove-integration', function () {
        let account = $(this).data('account');
        chrome.storage.local.get('tokens', function (tokens) {
          if (tokens.tokens !== undefined) {
            tokens = tokens.tokens;

            delete tokens[account];

            checkIntegrations(tokens);

            chrome.storage.local.set({'tokens': tokens}, function () {
              showIntegrations();
            });
          }
        });
      });

      $(document).on('click', '.extension-control .control-enable', function () {
        enableExtension();
      });

      $(document).on('click', '.extension-control .control-disable', function () {
        disableExtension();
      });
    }
  })
};
initPopUp();
const updateToken = response => {
  if (response.messageType && response.messageType === 'CLOSE_INTEGRATION_INVITATION_POPUP' && response.token) {
    chrome.runtime.sendMessage({messageType: 'GET_HOST'}, (res) => {
      if (res.host.length !== 0) {
        const dataToken = {
          tokens: {}
        };
        res.host.forEach(item => {
          dataToken.tokens[item] = response.token
        });
        chrome.storage.local.set(dataToken, () => {
          $(document).off('click', '.wildix-integrations-wrapper .add-new-integration');
          $(document).off('click', '.wildix-integrations-wrapper .remove-integration');
          $(document).off('click', '.extension-control .control-enable');
          $(document).off('click', '.extension-control .control-disable');
          initPopUp()
        });
      }
    });
  }
};
chrome.runtime.onMessage.addListener(updateToken);
