class ControllerCallBtn {
  constructor(container, callType, channel, state, timerSection, timer) {
    this.container = container;
    this.callType = callType;
    this.channel = channel;
    this.callState = state;
    this.timerSection = timerSection;
    this.timer = timer;
    this.acceptBtn = $('<button class="btnCall btnAccept" title="Answer">' +
      '<i class="fa fa-phone" aria-hidden="true"></i>' +
      '</button>');
    this.declineBtn = $('<button class="btnCall btnDecline" title="Decline">' +
      '<i class="fa fa-phone" aria-hidden="true"></i>' +
      '</button>');
    this.pauseBtn = $('<button class="btnCall btnPause" title="Hold">' +
      '<i class="fa fa-pause" aria-hidden="true"></i>' +
      '</button>');
    this.init();
  }
  init() {
    this.getCollaborationVersion().then(v => {
      const version = v.split('.')[0];
      if (parseInt(version) < 4) {
        return false;
      }
      if (this.callState !== 'ring') {
        this.state = this.callState;
      }
      this.renderBtn();
      this.registerEvent();
    }).catch(e => {
      console.error(e);
    });
  }
  renderBtn() {
    if (this.callType === 'incoming') {
      this.container.append(this.acceptBtn);
      this.container.append(this.declineBtn);
    } else {
      this.container.append(this.declineBtn);
    }
  }
  acceptCall(target, channel) {
    this.sendMessage({ messageType: 'ACCEPT_CALL', channel: channel });
    this.container.append(this.pauseBtn);
    target.hide();
  }
  declineCall(target, channel) {
    this.sendMessage({ messageType: 'DECLINE_CALL', channel: channel });
    target.parent().hide();
  }
  pauseCall(channel) {
    this.sendMessage({ messageType: 'PAUSE_CALL', channel: channel });
  }
  unPauseCall(channel) {
    this.sendMessage({ messageType: 'REMOVE_PAUSE_CALL', channel: channel });
  }
  registerEvent() {
    this.acceptBtn.on('click', (event) => {
      this.acceptCall(event.target.tagName === 'I' ? $(event.target).parent() : $(event.target), this.channel);
    });
    this.declineBtn.on('click', (event) => {
      this.declineCall($(event.target), this.channel);
    });
    this.pauseBtn.on('click', () => {
      if (this.state === 'up') {
        this.pauseCall(this.channel)
      } else if (this.state === 'hold') {
        this.unPauseCall(this.channel)
      }
    });
    this.timer.addEventListener('secondsUpdated', e => {
      this.timerSection.html(this.timer.getTimeValues().toString());
    });
  }
  getCollaborationVersion() {
    const promise = () => {
      return new Promise((resolve, reject) => {
        chrome.runtime.sendMessage({messageType: 'GET_COLLABORATION_VERSION'}, (version) => {
          if (version) {
            resolve(version);
          } else {
            reject(new Error('failed get collaboration version'));
          }
        });
      })
    };
    return promise().then(v => v);
  }
  get state () {
    return this.callState;
  }
  set state (val) {
    if (val === 'up') {
      if (!this.timer.isRunning()) {
        this.timer.start();
      }
      this.renderHoldBtn();
      this.pauseBtn.children().removeClass('blinkPauseCallBtn');
      this.pauseBtn.attr('title', 'Hold');
      this.acceptBtn.hide();
    } else if (val === 'hold') {
      this.acceptBtn.hide();
      this.renderHoldBtn();
      this.pauseBtn.attr('title', 'Resume');
      this.pauseBtn.children().addClass('blinkPauseCallBtn');
    } else if (val === 'remove') {
      this.timer.reset();
      this.destroyController();
    }
    this.callState = val;
  }
  sendMessage (data) {
    chrome.runtime.sendMessage(data);
  }
  renderHoldBtn () {
    if (!this.container.is(this.pauseBtn)) {
      this.container.append(this.pauseBtn);
    }
  }
  destroyController () {
    this.container.remove();
    this.timerSection.replaceWith('<div class="emptyTimer"></div>');
  }
}